<?php

namespace Drupal\micro_sso\StackMiddleware;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\micro_site\SiteNegotiatorInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Middleware for the micro_site module.
 */
class MicroSsoMiddleware implements HttpKernelInterface {

  /**
   * The decorated kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Site negotiator.
   *
   * @var \Drupal\micro_site\SiteNegotiatorInterface
   */
  protected $negotiator;

  /**
   * Constructs a StackMiddleware object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $http_kernel
   *   The decorated kernel.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\micro_site\SiteNegotiatorInterface $site_negotiator
   *   The Site negotiator.
   */
  public function __construct(HttpKernelInterface $http_kernel, ConfigFactoryInterface $config_factory, SiteNegotiatorInterface $site_negotiator) {
    $this->httpKernel = $http_kernel;
    $this->configFactory = $config_factory;
    $this->negotiator = $site_negotiator;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MASTER_REQUEST, $catch = TRUE) {
    if ($this->isMaster()) {
      $request->headers->set('Access-Control-Allow-Origin', $origin);
      $request->headers->set('Access-Control-Allow-Credentials', 'true');
      $request->headers->set('Access-Control-Allow-Headers', 'Accept, Content-Type');
      $request->headers->set('Access-Control-Allow-Methods', 'OPTIONS, GET, POST');
      if ($origin = $this->getOrigin($request)) {
        // we had an origin in the request query, should not happen with images
        // or classical pages so we can be quite sure this was an ajax request.
        // We need to allow modern browsers to send cookies on theses ajax cross
        // domain requests.
        // Note that Access-Control-Allow-Origin: * would not work with
        // credentials, so we also need to send a domain specific authorization.
        // The micro_sso_origin_get() made the check about validity of this origin.
        $request->headers->set('Access-Control-Allow-Origin', $origin);
        $request->headers->set('Access-Control-Allow-Credentials', 'true');
        $request->headers->set('Access-Control-Allow-Headers', 'Accept, Content-Type');
        $request->headers->set('Access-Control-Allow-Methods', 'OPTIONS, GET, POST');
      }
    }
    return $this->httpKernel->handle($request, $type, $catch);
  }

  public function isMaster() {
    return !$this->negotiator->getActiveSite() && !(PHP_SAPI === 'cli') && $_SERVER['HTTP_HOST'] === $this->configFactory->get('micro_site.settings')->get('public_url');
  }

  /**
   * Get a valid HTTP Origin header if it matches an existing site.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return boolean
   */
  public function getOrigin(Request $request) {
    $http_origin = $request->server->get('HTTP_ORIGIN');
    if (!empty($http_origin)) {
      $url = parse_url($http_origin);
      /** @var \Drupal\micro_site\Entity\SiteInterface $site */
      $site = $this->negotiator->loadByHostname($url['host']);
      if ($site) { // @TODO check user access to site too.
        return $site->getSitePath();
      }
    }
    return NULL;
  }

}
